@@ -1,7 +1,7 @@
 /*
  * accounts.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -60,6 +60,7 @@ static gchar *string_keys[] = {
     "port",
     "resource",
     "password",
+    "eval_password",
     "presence.last",
     "presence.login",
     "muc.service",
@@ -73,8 +74,8 @@ static gchar * _get_accounts_file(void);
 static void _remove_from_list(GKeyFile *accounts, const char * const account_name, const char * const key, const char * const contact_jid);
 
 
-static void
-_accounts_load(void)
+void
+accounts_load(void)
 {
     log_info("Loading accounts");
     all_ac = autocomplete_new();
@@ -107,40 +108,40 @@ _accounts_load(void)
     g_strfreev(account_names);
 }
 
-static void
-_accounts_close(void)
+void
+accounts_close(void)
 {
     autocomplete_free(all_ac);
     autocomplete_free(enabled_ac);
     g_key_file_free(accounts);
 }
 
-static char *
-_accounts_find_enabled(char *prefix)
+char *
+accounts_find_enabled(const char * const prefix)
 {
     return autocomplete_complete(enabled_ac, prefix, TRUE);
 }
 
-static char *
-_accounts_find_all(char *prefix)
+char *
+accounts_find_all(const char * const prefix)
 {
     return autocomplete_complete(all_ac, prefix, TRUE);
 }
 
-static void
-_accounts_reset_all_search(void)
+void
+accounts_reset_all_search(void)
 {
     autocomplete_reset(all_ac);
 }
 
-static void
-_accounts_reset_enabled_search(void)
+void
+accounts_reset_enabled_search(void)
 {
     autocomplete_reset(enabled_ac);
 }
 
-static void
-_accounts_add(const char *account_name, const char *altdomain, const int port)
+void
+accounts_add(const char *account_name, const char *altdomain, const int port)
 {
     // set account name and resource
     const char *barejid = account_name;
@@ -193,14 +194,24 @@ _accounts_add(const char *account_name, const char *altdomain, const int port)
     jid_destroy(jid);
 }
 
-static gchar**
-_accounts_get_list(void)
+int
+accounts_remove(const char *account_name)
+{
+    int r = g_key_file_remove_group(accounts, account_name, NULL);
+    _save_accounts();
+    autocomplete_remove(all_ac, account_name);
+    autocomplete_remove(enabled_ac, account_name);
+    return r;
+}
+
+gchar**
+accounts_get_list(void)
 {
     return g_key_file_get_groups(accounts, NULL);
 }
 
-static ProfAccount*
-_accounts_get_account(const char * const name)
+ProfAccount*
+accounts_get_account(const char * const name)
 {
     if (!g_key_file_has_group(accounts, name)) {
         return NULL;
@@ -214,6 +225,7 @@ _accounts_get_account(const char * const name)
         }
 
         gchar *password = g_key_file_get_string(accounts, name, "password", NULL);
+        gchar *eval_password = g_key_file_get_string(accounts, name, "eval_password", NULL);
         gboolean enabled = g_key_file_get_boolean(accounts, name, "enabled", NULL);
 
         gchar *server = g_key_file_get_string(accounts, name, "server", NULL);
@@ -268,7 +280,7 @@ _accounts_get_account(const char * const name)
             g_strfreev(always);
         }
 
-        ProfAccount *new_account = account_new(name, jid, password, enabled,
+        ProfAccount *new_account = account_new(name, jid, password, eval_password, enabled,
             server, port, resource, last_presence, login_presence,
             priority_online, priority_chat, priority_away, priority_xa,
             priority_dnd, muc_service, muc_nick, otr_policy, otr_manual,
@@ -276,6 +288,7 @@ _accounts_get_account(const char * const name)
 
         g_free(jid);
         g_free(password);
+        g_free(eval_password);
         g_free(server);
         g_free(resource);
         g_free(last_presence);
@@ -288,8 +301,8 @@ _accounts_get_account(const char * const name)
     }
 }
 
-static gboolean
-_accounts_enable(const char * const name)
+gboolean
+accounts_enable(const char * const name)
 {
     if (g_key_file_has_group(accounts, name)) {
         g_key_file_set_boolean(accounts, name, "enabled", TRUE);
@@ -301,8 +314,8 @@ _accounts_enable(const char * const name)
     }
 }
 
-static gboolean
-_accounts_disable(const char * const name)
+gboolean
+accounts_disable(const char * const name)
 {
     if (g_key_file_has_group(accounts, name)) {
         g_key_file_set_boolean(accounts, name, "enabled", FALSE);
@@ -314,8 +327,8 @@ _accounts_disable(const char * const name)
     }
 }
 
-static gboolean
-_accounts_rename(const char * const account_name, const char * const new_name)
+gboolean
+accounts_rename(const char * const account_name, const char * const new_name)
 {
     if (g_key_file_has_group(accounts, new_name)) {
         return FALSE;
@@ -362,15 +375,15 @@ _accounts_rename(const char * const account_name, const char * const new_name)
     return TRUE;
 }
 
-static gboolean
-_accounts_account_exists(const char * const account_name)
+gboolean
+accounts_account_exists(const char * const account_name)
 {
     return g_key_file_has_group(accounts, account_name);
 
 }
 
-static void
-_accounts_set_jid(const char * const account_name, const char * const value)
+void
+accounts_set_jid(const char * const account_name, const char * const value)
 {
     Jid *jid = jid_create(value);
     if (jid != NULL) {
@@ -395,8 +408,8 @@ _accounts_set_jid(const char * const account_name, const char * const value)
     }
 }
 
-static void
-_accounts_set_server(const char * const account_name, const char * const value)
+void
+accounts_set_server(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "server", value);
@@ -404,8 +417,8 @@ _accounts_set_server(const char * const account_name, const char * const value)
     }
 }
 
-static void
-_accounts_set_port(const char * const account_name, const int value)
+void
+accounts_set_port(const char * const account_name, const int value)
 {
     if (value != 0) {
         g_key_file_set_integer(accounts, account_name, "port", value);
@@ -413,8 +426,8 @@ _accounts_set_port(const char * const account_name, const int value)
     }
 }
 
-static void
-_accounts_set_resource(const char * const account_name, const char * const value)
+void
+accounts_set_resource(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "resource", value);
@@ -422,8 +435,8 @@ _accounts_set_resource(const char * const account_name, const char * const value
     }
 }
 
-static void
-_accounts_set_password(const char * const account_name, const char * const value)
+void
+accounts_set_password(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "password", value);
@@ -431,8 +444,17 @@ _accounts_set_password(const char * const account_name, const char * const value
     }
 }
 
-static void
-_accounts_clear_password(const char * const account_name)
+void
+accounts_set_eval_password(const char * const account_name, const char * const value)
+{
+    if (accounts_account_exists(account_name)) {
+        g_key_file_set_string(accounts, account_name, "eval_password", value);
+        _save_accounts();
+    }
+}
+
+void
+accounts_clear_password(const char * const account_name)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_remove_key(accounts, account_name, "password", NULL);
@@ -440,8 +462,17 @@ _accounts_clear_password(const char * const account_name)
     }
 }
 
-static void
-_accounts_clear_server(const char * const account_name)
+void
+accounts_clear_eval_password(const char * const account_name)
+{
+    if (accounts_account_exists(account_name)) {
+        g_key_file_remove_key(accounts, account_name, "eval_password", NULL);
+        _save_accounts();
+    }
+}
+
+void
+accounts_clear_server(const char * const account_name)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_remove_key(accounts, account_name, "server", NULL);
@@ -449,8 +480,8 @@ _accounts_clear_server(const char * const account_name)
     }
 }
 
-static void
-_accounts_clear_port(const char * const account_name)
+void
+accounts_clear_port(const char * const account_name)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_remove_key(accounts, account_name, "port", NULL);
@@ -458,8 +489,8 @@ _accounts_clear_port(const char * const account_name)
     }
 }
 
-static void
-_accounts_clear_otr(const char * const account_name)
+void
+accounts_clear_otr(const char * const account_name)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_remove_key(accounts, account_name, "otr.policy", NULL);
@@ -467,8 +498,8 @@ _accounts_clear_otr(const char * const account_name)
     }
 }
 
-static void
-_accounts_add_otr_policy(const char * const account_name, const char * const contact_jid, const char * const policy)
+void
+accounts_add_otr_policy(const char * const account_name, const char * const contact_jid, const char * const policy)
 {
     if (accounts_account_exists(account_name)) {
         GString *key = g_string_new("otr.");
@@ -579,8 +610,8 @@ _remove_from_list(GKeyFile *accounts, const char * const account_name, const cha
     g_strfreev(list);
 }
 
-static void
-_accounts_set_muc_service(const char * const account_name, const char * const value)
+void
+accounts_set_muc_service(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "muc.service", value);
@@ -588,8 +619,8 @@ _accounts_set_muc_service(const char * const account_name, const char * const va
     }
 }
 
-static void
-_accounts_set_muc_nick(const char * const account_name, const char * const value)
+void
+accounts_set_muc_nick(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "muc.nick", value);
@@ -597,8 +628,8 @@ _accounts_set_muc_nick(const char * const account_name, const char * const value
     }
 }
 
-static void
-_accounts_set_otr_policy(const char * const account_name, const char * const value)
+void
+accounts_set_otr_policy(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "otr.policy", value);
@@ -606,8 +637,8 @@ _accounts_set_otr_policy(const char * const account_name, const char * const val
     }
 }
 
-static void
-_accounts_set_priority_online(const char * const account_name, const gint value)
+void
+accounts_set_priority_online(const char * const account_name, const gint value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_integer(accounts, account_name, "priority.online", value);
@@ -615,8 +646,8 @@ _accounts_set_priority_online(const char * const account_name, const gint value)
     }
 }
 
-static void
-_accounts_set_priority_chat(const char * const account_name, const gint value)
+void
+accounts_set_priority_chat(const char * const account_name, const gint value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_integer(accounts, account_name, "priority.chat", value);
@@ -624,8 +655,8 @@ _accounts_set_priority_chat(const char * const account_name, const gint value)
     }
 }
 
-static void
-_accounts_set_priority_away(const char * const account_name, const gint value)
+void
+accounts_set_priority_away(const char * const account_name, const gint value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_integer(accounts, account_name, "priority.away", value);
@@ -633,8 +664,8 @@ _accounts_set_priority_away(const char * const account_name, const gint value)
     }
 }
 
-static void
-_accounts_set_priority_xa(const char * const account_name, const gint value)
+void
+accounts_set_priority_xa(const char * const account_name, const gint value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_integer(accounts, account_name, "priority.xa", value);
@@ -642,8 +673,8 @@ _accounts_set_priority_xa(const char * const account_name, const gint value)
     }
 }
 
-static void
-_accounts_set_priority_dnd(const char * const account_name, const gint value)
+void
+accounts_set_priority_dnd(const char * const account_name, const gint value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_integer(accounts, account_name, "priority.dnd", value);
@@ -651,8 +682,8 @@ _accounts_set_priority_dnd(const char * const account_name, const gint value)
     }
 }
 
-static void
-_accounts_set_priority_all(const char * const account_name, const gint value)
+void
+accounts_set_priority_all(const char * const account_name, const gint value)
 {
     if (accounts_account_exists(account_name)) {
         accounts_set_priority_online(account_name, value);
@@ -664,8 +695,8 @@ _accounts_set_priority_all(const char * const account_name, const gint value)
     }
 }
 
-static gint
-_accounts_get_priority_for_presence_type(const char * const account_name,
+gint
+accounts_get_priority_for_presence_type(const char * const account_name,
     resource_presence_t presence_type)
 {
     gint result;
@@ -695,8 +726,8 @@ _accounts_get_priority_for_presence_type(const char * const account_name,
     return result;
 }
 
-static void
-_accounts_set_last_presence(const char * const account_name, const char * const value)
+void
+accounts_set_last_presence(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "presence.last", value);
@@ -704,8 +735,8 @@ _accounts_set_last_presence(const char * const account_name, const char * const
     }
 }
 
-static void
-_accounts_set_login_presence(const char * const account_name, const char * const value)
+void
+accounts_set_login_presence(const char * const account_name, const char * const value)
 {
     if (accounts_account_exists(account_name)) {
         g_key_file_set_string(accounts, account_name, "presence.login", value);
@@ -713,8 +744,8 @@ _accounts_set_login_presence(const char * const account_name, const char * const
     }
 }
 
-static resource_presence_t
-_accounts_get_last_presence(const char * const account_name)
+resource_presence_t
+accounts_get_last_presence(const char * const account_name)
 {
     resource_presence_t result;
     gchar *setting = g_key_file_get_string(accounts, account_name, "presence.last", NULL);
@@ -741,8 +772,8 @@ _accounts_get_last_presence(const char * const account_name)
     return result;
 }
 
-static resource_presence_t
-_accounts_get_login_presence(const char * const account_name)
+resource_presence_t
+accounts_get_login_presence(const char * const account_name)
 {
     resource_presence_t result;
     gchar *setting = g_key_file_get_string(accounts, account_name, "presence.login", NULL);
@@ -797,7 +828,7 @@ _fix_legacy_accounts(const char * const account_name)
         _save_accounts();
     }
 
-    // acounts with no muc service or nick
+    // accounts with no muc service or nick
     if (!g_key_file_has_key(accounts, account_name, "muc.service", NULL)) {
         gchar *account_jid = g_key_file_get_string(accounts, account_name, "jid", NULL);
         Jid *jidp = jid_create(account_jid);
@@ -850,46 +881,3 @@ _get_accounts_file(void)
 
     return result;
 }
-
-void
-accounts_init_module(void)
-{
-    accounts_load = _accounts_load;
-    accounts_close = _accounts_close;
-    accounts_find_all = _accounts_find_all;
-    accounts_find_enabled = _accounts_find_enabled;
-    accounts_reset_all_search = _accounts_reset_all_search;
-    accounts_reset_enabled_search = _accounts_reset_enabled_search;
-    accounts_add = _accounts_add;
-    accounts_get_list = _accounts_get_list;
-    accounts_get_account = _accounts_get_account;
-    accounts_enable = _accounts_enable;
-    accounts_disable = _accounts_disable;
-    accounts_rename = _accounts_rename;
-    accounts_account_exists = _accounts_account_exists;
-    accounts_set_jid = _accounts_set_jid;
-    accounts_set_server = _accounts_set_server;
-    accounts_set_port = _accounts_set_port;
-    accounts_set_resource = _accounts_set_resource;
-    accounts_set_password = _accounts_set_password;
-    accounts_set_muc_service = _accounts_set_muc_service;
-    accounts_set_muc_nick = _accounts_set_muc_nick;
-    accounts_set_otr_policy = _accounts_set_otr_policy;
-    accounts_set_last_presence = _accounts_set_last_presence;
-    accounts_set_login_presence = _accounts_set_login_presence;
-    accounts_get_last_presence = _accounts_get_last_presence;
-    accounts_get_login_presence = _accounts_get_login_presence;
-    accounts_set_priority_online = _accounts_set_priority_online;
-    accounts_set_priority_chat = _accounts_set_priority_chat;
-    accounts_set_priority_away = _accounts_set_priority_away;
-    accounts_set_priority_xa = _accounts_set_priority_xa;
-    accounts_set_priority_dnd = _accounts_set_priority_dnd;
-    accounts_set_priority_all = _accounts_set_priority_all;
-    accounts_get_priority_for_presence_type = _accounts_get_priority_for_presence_type;
-    accounts_clear_password = _accounts_clear_password;
-    accounts_clear_server = _accounts_clear_server;
-    accounts_clear_port = _accounts_clear_port;
-    accounts_clear_otr = _accounts_clear_otr;
-    accounts_add_otr_policy = _accounts_add_otr_policy;
-}
-
