@@ -1,7 +1,7 @@
 /*
  * muc.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -42,6 +42,7 @@
 #include "jid.h"
 #include "tools/autocomplete.h"
 #include "ui/ui.h"
+#include "ui/windows.h"
 #include "muc.h"
 
 typedef struct _muc_room_t {
@@ -140,7 +141,7 @@ muc_invites_reset_ac(void)
 }
 
 char *
-muc_invites_find(char *search_str)
+muc_invites_find(const char * const search_str)
 {
     return autocomplete_complete(invite_ac, search_str, TRUE);
 }
@@ -314,7 +315,7 @@ muc_nick_change_pending(const char * const room)
 }
 
 /*
- * Change the current nuck name for the room, call once
+ * Change the current nick name for the room, call once
  * the service has responded
  */
 void
@@ -631,41 +632,46 @@ muc_roster_nick_change_complete(const char * const room,
     return NULL;
 }
 
-void
-muc_autocomplete(char *input, int *size)
+char *
+muc_autocomplete(const char * const input)
 {
-    char *recipient = ui_current_recipient();
-    ChatRoom *chat_room = g_hash_table_lookup(rooms, recipient);
+    win_type_t wintype = ui_current_win_type();
+    if (wintype == WIN_MUC) {
+        ProfMucWin *mucwin = wins_get_current_muc();
+        ChatRoom *chat_room = g_hash_table_lookup(rooms, mucwin->roomjid);
 
-    if (chat_room && chat_room->nick_ac) {
-        input[*size] = '\0';
-        char *search_str = NULL;
+        if (chat_room && chat_room->nick_ac) {
+            const char * search_str = NULL;
 
-        gchar *last_space = g_strrstr(input, " ");
-        if (!last_space) {
-            search_str = input;
-            if (!chat_room->autocomplete_prefix) {
-                chat_room->autocomplete_prefix = strdup("");
+            gchar *last_space = g_strrstr(input, " ");
+            if (!last_space) {
+                search_str = input;
+                if (!chat_room->autocomplete_prefix) {
+                    chat_room->autocomplete_prefix = strdup("");
+                }
+            } else {
+                search_str = last_space+1;
+                if (!chat_room->autocomplete_prefix) {
+                    chat_room->autocomplete_prefix = g_strndup(input, search_str - input);
+                }
             }
-        } else {
-            search_str = last_space+1;
-            if (!chat_room->autocomplete_prefix) {
-                chat_room->autocomplete_prefix = g_strndup(input, search_str - input);
-            }
-        }
 
-        char *result = autocomplete_complete(chat_room->nick_ac, search_str, FALSE);
-        if (result) {
-            GString *replace_with = g_string_new(chat_room->autocomplete_prefix);
-            g_string_append(replace_with, result);
-            if (!last_space || (*(last_space+1) == '\0')) {
-                g_string_append(replace_with, ": ");
+            char *result = autocomplete_complete(chat_room->nick_ac, search_str, FALSE);
+            if (result) {
+                GString *replace_with = g_string_new(chat_room->autocomplete_prefix);
+                g_string_append(replace_with, result);
+                if (!last_space || (*(last_space+1) == '\0')) {
+                    g_string_append(replace_with, ": ");
+                }
+                g_free(result);
+                result = replace_with->str;
+                g_string_free(replace_with, FALSE);
+                return result;
             }
-            ui_replace_input(input, replace_with->str, size);
-            g_string_free(replace_with, TRUE);
-            g_free(result);
         }
     }
+
+    return NULL;
 }
 
 void
