@@ -1,7 +1,7 @@
 /*
  * muc.h
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -114,7 +114,7 @@ gint muc_invites_count(void);
 GSList* muc_invites(void);
 gboolean muc_invites_contain(const char * const room);
 void muc_invites_reset_ac(void);
-char* muc_invites_find(char *search_str);
+char* muc_invites_find(const char * const search_str);
 void muc_invites_clear(void);
 
 void muc_set_subject(const char * const room, const char * const subject);
@@ -123,7 +123,7 @@ char* muc_subject(const char * const room);
 void muc_pending_broadcasts_add(const char * const room, const char * const message);
 GList * muc_pending_broadcasts(const char * const room);
 
-void muc_autocomplete(char *input, int *size);
+char* muc_autocomplete(const char * const input);
 void muc_autocomplete_reset(const char * const room);
 
 gboolean muc_requires_config(const char * const room);
