@@ -1,7 +1,7 @@
 /*
  * parser.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -47,9 +47,9 @@
  *
  * inp - The line of input
  * min - The minimum allowed number of arguments
- * max - The maxmimum allowed number of arguments
+ * max - The maximum allowed number of arguments
  *
- * Returns - An NULL terminated array of strings representing the aguments
+ * Returns - An NULL terminated array of strings representing the arguments
  * of the command, or NULL if the validation fails.
  *
  * E.g. the following input line:
@@ -69,7 +69,7 @@ parse_args(const char * const inp, int min, int max, gboolean *result)
         return NULL;
     }
 
-    // copy and strip input of leading/trailing whitepsace
+    // copy and strip input of leading/trailing whitespace
     char *copy = strdup(inp);
     g_strstrip(copy);
 
@@ -181,9 +181,9 @@ parse_args(const char * const inp, int min, int max, gboolean *result)
  *
  * inp - The line of input
  * min - The minimum allowed number of arguments
- * max - The maxmimum allowed number of arguments
+ * max - The maximum allowed number of arguments
  *
- * Returns - An NULL terminated array of strings representing the aguments
+ * Returns - An NULL terminated array of strings representing the arguments
  * of the command, or NULL if the validation fails.
  *
  * E.g. the following input line:
@@ -316,7 +316,7 @@ parse_args_with_freetext(const char * const inp, int min, int max, gboolean *res
 }
 
 int
-count_tokens(char *string)
+count_tokens(const char * const string)
 {
     int length = g_utf8_strlen(string, -1);
     gboolean in_quotes = FALSE;
@@ -347,7 +347,7 @@ count_tokens(char *string)
 }
 
 char *
-get_start(char *string, int tokens)
+get_start(const char * const string, int tokens)
 {
     GString *result = g_string_new("");
     int length = g_utf8_strlen(string, -1);
