@@ -1,7 +1,7 @@
 /*
  * buffer.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -80,14 +80,14 @@ buffer_free(ProfBuff buffer)
 }
 
 void
-buffer_push(ProfBuff buffer, const char show_char, const char * const date_fmt,
-    int flags, int attrs, const char * const from, const char * const message)
+buffer_push(ProfBuff buffer, const char show_char, GDateTime *time,
+    int flags, theme_item_t theme_item, const char * const from, const char * const message)
 {
     ProfBuffEntry *e = malloc(sizeof(struct prof_buff_entry_t));
     e->show_char = show_char;
     e->flags = flags;
-    e->attrs = attrs;
-    e->date_fmt = strdup(date_fmt);
+    e->theme_item = theme_item;
+    e->time = time;
     e->from = strdup(from);
     e->message = strdup(message);
 
@@ -111,7 +111,6 @@ _free_entry(ProfBuffEntry *entry)
 {
     free(entry->message);
     free(entry->from);
-    free(entry->date_fmt);
+    g_date_time_unref(entry->time);
     free(entry);
-}
-
+}
\ No newline at end of file
