@@ -1,7 +1,7 @@
 /*
  * bookmark.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -50,10 +50,6 @@
 #include "ui/ui.h"
 
 #define BOOKMARK_TIMEOUT 5000
-/* TODO: replace with a preference */
-#define BOOKMARK_AUTOJOIN_MAX 5
-
-static int autojoin_count;
 
 static Autocomplete bookmark_ac;
 static GList *bookmark_list;
@@ -76,7 +72,6 @@ bookmark_request(void)
 
     id = strdup("bookmark_init_request");
 
-    autojoin_count = 0;
     autocomplete_free(bookmark_ac);
     bookmark_ac = autocomplete_new();
     if (bookmark_list != NULL) {
@@ -93,8 +88,8 @@ bookmark_request(void)
     xmpp_stanza_release(iq);
 }
 
-static gboolean
-_bookmark_add(const char *jid, const char *nick, const char *password, const char *autojoin_str)
+gboolean
+bookmark_add(const char *jid, const char *nick, const char *password, const char *autojoin_str)
 {
     if (autocomplete_contains(bookmark_ac, jid)) {
         return FALSE;
@@ -126,8 +121,8 @@ _bookmark_add(const char *jid, const char *nick, const char *password, const cha
     }
 }
 
-static gboolean
-_bookmark_update(const char *jid, const char *nick, const char *password, const char *autojoin_str)
+gboolean
+bookmark_update(const char *jid, const char *nick, const char *password, const char *autojoin_str)
 {
     Bookmark *item = malloc(sizeof(*item));
     item->jid = strdup(jid);
@@ -161,8 +156,8 @@ _bookmark_update(const char *jid, const char *nick, const char *password, const
     }
 }
 
-static gboolean
-_bookmark_join(const char *jid)
+gboolean
+bookmark_join(const char *jid)
 {
     Bookmark *item = malloc(sizeof(*item));
     item->jid = strdup(jid);
@@ -193,8 +188,8 @@ _bookmark_join(const char *jid)
     }
 }
 
-static gboolean
-_bookmark_remove(const char *jid)
+gboolean
+bookmark_remove(const char *jid)
 {
     Bookmark *item = malloc(sizeof(*item));
     item->jid = strdup(jid);
@@ -218,20 +213,20 @@ _bookmark_remove(const char *jid)
     }
 }
 
-static const GList *
-_bookmark_get_list(void)
+const GList *
+bookmark_get_list(void)
 {
     return bookmark_list;
 }
 
-static char *
-_bookmark_find(char *search_str)
+char *
+bookmark_find(const char * const search_str)
 {
     return autocomplete_complete(bookmark_ac, search_str, TRUE);
 }
 
-static void
-_bookmark_autocomplete_reset(void)
+void
+bookmark_autocomplete_reset(void)
 {
     if (bookmark_ac != NULL) {
         autocomplete_reset(bookmark_ac);
@@ -330,28 +325,22 @@ _bookmark_handle_result(xmpp_conn_t * const conn,
         bookmark_list = g_list_append(bookmark_list, item);
 
         if (autojoin_val) {
-            if (autojoin_count < BOOKMARK_AUTOJOIN_MAX) {
-                Jid *room_jid;
+            Jid *room_jid;
 
-                ++autojoin_count;
-
-                char *account_name = jabber_get_account_name();
-                ProfAccount *account = accounts_get_account(account_name);
-                if (name == NULL) {
-                    name = account->muc_nick;
-                }
-
-                log_debug("Autojoin %s with nick=%s", jid, name);
-                room_jid = jid_create_from_bare_and_resource(jid, name);
-                if (!muc_active(room_jid->barejid)) {
-                    presence_join_room(jid, name, password);
-                    muc_join(jid, name, password, TRUE);
-                }
-                jid_destroy(room_jid);
-                account_free(account);
-            } else {
-                log_debug("Rejected autojoin %s (maximum has been reached)", jid);
+            char *account_name = jabber_get_account_name();
+            ProfAccount *account = accounts_get_account(account_name);
+            if (name == NULL) {
+                name = account->muc_nick;
             }
+
+            log_debug("Autojoin %s with nick=%s", jid, name);
+            room_jid = jid_create_from_bare_and_resource(jid, name);
+            if (!muc_active(room_jid->barejid)) {
+                presence_join_room(jid, name, password);
+                muc_join(jid, name, password, TRUE);
+            }
+            jid_destroy(room_jid);
+            account_free(account);
         }
 
         ptr = xmpp_stanza_get_next(ptr);
@@ -479,16 +468,4 @@ _send_bookmarks(void)
 
     xmpp_send(conn, iq);
     xmpp_stanza_release(iq);
-}
-
-void
-bookmark_init_module(void)
-{
-    bookmark_add = _bookmark_add;
-    bookmark_update = _bookmark_update;
-    bookmark_remove = _bookmark_remove;
-    bookmark_join = _bookmark_join;
-    bookmark_get_list = _bookmark_get_list;
-    bookmark_find = _bookmark_find;
-    bookmark_autocomplete_reset = _bookmark_autocomplete_reset;
-}
+}
\ No newline at end of file
