@@ -1,7 +1,7 @@
 /*
  * connection.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -104,8 +104,8 @@ void _connection_free_saved_account(void);
 void _connection_free_saved_details(void);
 void _connection_free_session_data(void);
 
-static void
-_jabber_init(const int disable_tls)
+void
+jabber_init(const int disable_tls)
 {
     log_info("Initialising XMPP");
     jabber_conn.conn_status = JABBER_STARTED;
@@ -121,8 +121,8 @@ _jabber_init(const int disable_tls)
     xmpp_initialize();
 }
 
-static jabber_conn_status_t
-_jabber_connect_with_account(const ProfAccount * const account)
+jabber_conn_status_t
+jabber_connect_with_account(const ProfAccount * const account)
 {
     assert(account != NULL);
 
@@ -147,8 +147,8 @@ _jabber_connect_with_account(const ProfAccount * const account)
     return result;
 }
 
-static jabber_conn_status_t
-_jabber_connect_with_details(const char * const jid,
+jabber_conn_status_t
+jabber_connect_with_details(const char * const jid,
     const char * const passwd, const char * const altdomain, const int port)
 {
     assert(jid != NULL);
@@ -184,8 +184,8 @@ _jabber_connect_with_details(const char * const jid,
     return _jabber_connect(saved_details.jid, passwd, saved_details.altdomain, saved_details.port);
 }
 
-static void
-_jabber_disconnect(void)
+void
+jabber_disconnect(void)
 {
     // if connected, send end stream and wait for response
     if (jabber_conn.conn_status == JABBER_CONNECTED) {
@@ -214,8 +214,8 @@ _jabber_disconnect(void)
     FREE_SET_NULL(jabber_conn.domain);
 }
 
-static void
-_jabber_shutdown(void)
+void
+jabber_shutdown(void)
 {
     _connection_free_saved_account();
     _connection_free_saved_details();
@@ -224,8 +224,8 @@ _jabber_shutdown(void)
     free(jabber_conn.log);
 }
 
-static void
-_jabber_process_events(void)
+void
+jabber_process_events(void)
 {
     int reconnect_sec;
 
@@ -250,14 +250,14 @@ _jabber_process_events(void)
     }
 }
 
-static GList *
-_jabber_get_available_resources(void)
+GList *
+jabber_get_available_resources(void)
 {
     return g_hash_table_get_values(available_resources);
 }
 
-static jabber_conn_status_t
-_jabber_get_connection_status(void)
+jabber_conn_status_t
+jabber_get_connection_status(void)
 {
     return (jabber_conn.conn_status);
 }
@@ -274,26 +274,26 @@ connection_get_ctx(void)
     return jabber_conn.ctx;
 }
 
-static const char *
-_jabber_get_fulljid(void)
+const char *
+jabber_get_fulljid(void)
 {
     return xmpp_conn_get_jid(jabber_conn.conn);
 }
 
-static const char *
-_jabber_get_domain(void)
+const char *
+jabber_get_domain(void)
 {
     return jabber_conn.domain;
 }
 
-static char *
-_jabber_get_presence_message(void)
+char *
+jabber_get_presence_message(void)
 {
     return jabber_conn.presence_message;
 }
 
-static char *
-_jabber_get_account_name(void)
+char *
+jabber_get_account_name(void)
 {
     return saved_account.name;
 }
@@ -572,21 +572,4 @@ _xmpp_get_file_logger()
     file_log->userdata = &level;
 
     return file_log;
-}
-
-void
-jabber_init_module(void)
-{
-    jabber_init = _jabber_init;
-    jabber_connect_with_account = _jabber_connect_with_account;
-    jabber_connect_with_details = _jabber_connect_with_details;
-    jabber_disconnect = _jabber_disconnect;
-    jabber_shutdown = _jabber_shutdown;
-    jabber_process_events = _jabber_process_events;
-    jabber_get_available_resources = _jabber_get_available_resources;
-    jabber_get_connection_status = _jabber_get_connection_status;
-    jabber_get_fulljid = _jabber_get_fulljid;
-    jabber_get_domain = _jabber_get_domain;
-    jabber_get_presence_message = _jabber_get_presence_message;
-    jabber_get_account_name = _jabber_get_account_name;
-}
+}
\ No newline at end of file
