@@ -1,7 +1,7 @@
 /*
  * form.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -379,8 +379,8 @@ _free_field(FormField *field)
     }
 }
 
-static void
-_form_destroy(DataForm *form)
+void
+form_destroy(DataForm *form)
 {
     if (form) {
         free(form->type);
@@ -400,8 +400,8 @@ _field_compare_by_var(FormField *a, FormField *b)
     return g_strcmp0(a->var, b->var);
 }
 
-static GSList *
-_form_get_non_form_type_fields_sorted(DataForm *form)
+GSList *
+form_get_non_form_type_fields_sorted(DataForm *form)
 {
     GSList *sorted = NULL;
     GSList *curr = form->fields;
@@ -416,8 +416,8 @@ _form_get_non_form_type_fields_sorted(DataForm *form)
     return sorted;
 }
 
-static GSList *
-_form_get_field_values_sorted(FormField *field)
+GSList *
+form_get_field_values_sorted(FormField *field)
 {
     GSList *sorted = NULL;
     GSList *curr = field->values;
@@ -432,8 +432,8 @@ _form_get_field_values_sorted(FormField *field)
     return sorted;
 }
 
-static char *
-_form_get_form_type_field(DataForm *form)
+char *
+form_get_form_type_field(DataForm *form)
 {
     GSList *curr = form->fields;
     while (curr) {
@@ -447,8 +447,8 @@ _form_get_form_type_field(DataForm *form)
     return NULL;
 }
 
-static gboolean
-_form_tag_exists(DataForm *form, const char * const tag)
+gboolean
+form_tag_exists(DataForm *form, const char * const tag)
 {
     GList *tags = g_hash_table_get_keys(form->tag_to_var);
     GList *curr = tags;
@@ -463,8 +463,8 @@ _form_tag_exists(DataForm *form, const char * const tag)
     return FALSE;
 }
 
-static form_field_type_t
-_form_get_field_type(DataForm *form, const char * const tag)
+form_field_type_t
+form_get_field_type(DataForm *form, const char * const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -480,8 +480,8 @@ _form_get_field_type(DataForm *form, const char * const tag)
     return FIELD_UNKNOWN;
 }
 
-static void
-_form_set_value(DataForm *form, const char * const tag, char *value)
+void
+form_set_value(DataForm *form, const char * const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -505,8 +505,8 @@ _form_set_value(DataForm *form, const char * const tag, char *value)
     }
 }
 
-static void
-_form_add_value(DataForm *form, const char * const tag, char *value)
+void
+form_add_value(DataForm *form, const char * const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -530,8 +530,8 @@ _form_add_value(DataForm *form, const char * const tag, char *value)
     }
 }
 
-static gboolean
-_form_add_unique_value(DataForm *form, const char * const tag, char *value)
+gboolean
+form_add_unique_value(DataForm *form, const char * const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -561,8 +561,8 @@ _form_add_unique_value(DataForm *form, const char * const tag, char *value)
     return FALSE;
 }
 
-static gboolean
-_form_remove_value(DataForm *form, const char * const tag, char *value)
+gboolean
+form_remove_value(DataForm *form, const char * const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -591,8 +591,8 @@ _form_remove_value(DataForm *form, const char * const tag, char *value)
     return FALSE;
 }
 
-static gboolean
-_form_remove_text_multi_value(DataForm *form, const char * const tag, int index)
+gboolean
+form_remove_text_multi_value(DataForm *form, const char * const tag, int index)
 {
     index--;
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
@@ -623,8 +623,8 @@ _form_remove_text_multi_value(DataForm *form, const char * const tag, int index)
     return FALSE;
 }
 
-static int
-_form_get_value_count(DataForm *form, const char * const tag)
+int
+form_get_value_count(DataForm *form, const char * const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -645,8 +645,8 @@ _form_get_value_count(DataForm *form, const char * const tag)
     return 0;
 }
 
-static gboolean
-_form_field_contains_option(DataForm *form, const char * const tag, char *value)
+gboolean
+form_field_contains_option(DataForm *form, const char * const tag, char *value)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -670,8 +670,8 @@ _form_field_contains_option(DataForm *form, const char * const tag, char *value)
     return FALSE;
 }
 
-static FormField *
-_form_get_field_by_tag(DataForm *form, const char * const tag)
+FormField *
+form_get_field_by_tag(DataForm *form, const char * const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -687,8 +687,8 @@ _form_get_field_by_tag(DataForm *form, const char * const tag)
     return NULL;
 }
 
-static Autocomplete
-_form_get_value_ac(DataForm *form, const char * const tag)
+Autocomplete
+form_get_value_ac(DataForm *form, const char * const tag)
 {
     char *var = g_hash_table_lookup(form->tag_to_var, tag);
     if (var) {
@@ -704,8 +704,8 @@ _form_get_value_ac(DataForm *form, const char * const tag)
     return NULL;
 }
 
-static void
-_form_reset_autocompleters(DataForm *form)
+void
+form_reset_autocompleters(DataForm *form)
 {
     autocomplete_reset(form->tag_ac);
     GSList *curr_field = form->fields;
@@ -714,25 +714,4 @@ _form_reset_autocompleters(DataForm *form)
         autocomplete_reset(field->value_ac);
         curr_field = g_slist_next(curr_field);
     }
-}
-
-void
-form_init_module(void)
-{
-    form_destroy = _form_destroy;
-    form_get_form_type_field = _form_get_form_type_field;
-    form_get_field_type = _form_get_field_type;
-    form_set_value = _form_set_value;
-    form_add_unique_value = _form_add_unique_value;
-    form_add_value = _form_add_value;
-    form_remove_value = _form_remove_value;
-    form_remove_text_multi_value = _form_remove_text_multi_value;
-    form_field_contains_option = _form_field_contains_option;
-    form_tag_exists = _form_tag_exists;
-    form_get_value_count = _form_get_value_count;
-    form_get_value_ac = _form_get_value_ac;
-    form_get_field_by_tag = _form_get_field_by_tag;
-    form_reset_autocompleters = _form_reset_autocompleters;
-    form_get_non_form_type_fields_sorted = _form_get_non_form_type_fields_sorted;
-    form_get_field_values_sorted = _form_get_field_values_sorted;
-}
+}
\ No newline at end of file
