@@ -1,7 +1,7 @@
 /*
  * roster.c
  *
- * Copyright (C) 2012 - 2014 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2015 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -95,8 +95,8 @@ roster_request(void)
     xmpp_stanza_release(iq);
 }
 
-static void
-_roster_send_add_new(const char * const barejid, const char * const name)
+void
+roster_send_add_new(const char * const barejid, const char * const name)
 {
     xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
@@ -105,8 +105,8 @@ _roster_send_add_new(const char * const barejid, const char * const name)
     xmpp_stanza_release(iq);
 }
 
-static void
-_roster_send_remove(const char * const barejid)
+void
+roster_send_remove(const char * const barejid)
 {
     xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
@@ -115,8 +115,8 @@ _roster_send_remove(const char * const barejid)
     xmpp_stanza_release(iq);
 }
 
-static void
-_roster_send_name_change(const char * const barejid, const char * const new_name, GSList *groups)
+void
+roster_send_name_change(const char * const barejid, const char * const new_name, GSList *groups)
 {
     xmpp_conn_t * const conn = connection_get_conn();
     xmpp_ctx_t * const ctx = connection_get_ctx();
@@ -126,8 +126,8 @@ _roster_send_name_change(const char * const barejid, const char * const new_name
     xmpp_stanza_release(iq);
 }
 
-static void
-_roster_send_add_to_group(const char * const group, PContact contact)
+void
+roster_send_add_to_group(const char * const group, PContact contact)
 {
     GSList *groups = p_contact_groups(contact);
     GSList *new_groups = NULL;
@@ -171,8 +171,8 @@ _group_add_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
     return 0;
 }
 
-static void
-_roster_send_remove_from_group(const char * const group, PContact contact)
+void
+roster_send_remove_from_group(const char * const group, PContact contact)
 {
     GSList *groups = p_contact_groups(contact);
     GSList *new_groups = NULL;
@@ -241,6 +241,7 @@ _roster_set_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
     jid_destroy(my_jid);
 
     const char *barejid = xmpp_stanza_get_attribute(item, STANZA_ATTR_JID);
+    gchar *barejid_lower = g_utf8_strdown(barejid, -1);
     const char *name = xmpp_stanza_get_attribute(item, STANZA_ATTR_NAME);
     const char *sub = xmpp_stanza_get_attribute(item, STANZA_ATTR_SUBSCRIPTION);
     const char *ask = xmpp_stanza_get_attribute(item, STANZA_ATTR_ASK);
@@ -254,12 +255,12 @@ _roster_set_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
     if (g_strcmp0(sub, "remove") == 0) {
         // remove barejid and name
         if (name == NULL) {
-            name = barejid;
+            name = barejid_lower;
         }
 
-        roster_remove(name, barejid);
+        roster_remove(name, barejid_lower);
 
-        handle_roster_remove(barejid);
+        handle_roster_remove(barejid_lower);
 
     // otherwise update local roster
     } else {
@@ -273,17 +274,19 @@ _roster_set_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
         GSList *groups = _get_groups_from_item(item);
 
         // update the local roster
-        PContact contact = roster_get_contact(barejid);
+        PContact contact = roster_get_contact(barejid_lower);
         if (contact == NULL) {
-            gboolean added = roster_add(barejid, name, groups, sub, pending_out);
+            gboolean added = roster_add(barejid_lower, name, groups, sub, pending_out);
             if (added) {
-                handle_roster_add(barejid, name);
+                handle_roster_add(barejid_lower, name);
             }
         } else {
-            roster_update(barejid, name, groups, sub, pending_out);
+            handle_roster_update(barejid_lower, name, groups, sub, pending_out);
         }
     }
 
+    g_free(barejid_lower);
+
     return 1;
 }
 
@@ -300,6 +303,7 @@ _roster_result_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
 
         while (item != NULL) {
             const char *barejid = xmpp_stanza_get_attribute(item, STANZA_ATTR_JID);
+            gchar *barejid_lower = g_utf8_strdown(barejid, -1);
             const char *name = xmpp_stanza_get_attribute(item, STANZA_ATTR_NAME);
             const char *sub = xmpp_stanza_get_attribute(item, STANZA_ATTR_SUBSCRIPTION);
 
@@ -316,15 +320,18 @@ _roster_result_handler(xmpp_conn_t * const conn, xmpp_stanza_t * const stanza,
 
             GSList *groups = _get_groups_from_item(item);
 
-            gboolean added = roster_add(barejid, name, groups, sub, pending_out);
+            gboolean added = roster_add(barejid_lower, name, groups, sub, pending_out);
 
             if (!added) {
-                log_warning("Attempt to add contact twice: %s", barejid);
+                log_warning("Attempt to add contact twice: %s", barejid_lower);
             }
 
+            g_free(barejid_lower);
             item = xmpp_stanza_get_next(item);
         }
 
+        handle_roster_received();
+
         resource_presence_t conn_presence = accounts_get_login_presence(jabber_get_account_name());
         presence_update(conn_presence, NULL, 0);
     }
@@ -349,15 +356,4 @@ _get_groups_from_item(xmpp_stanza_t *item)
     }
 
     return groups;
-}
-
-void
-roster_init_module(void)
-{
-    roster_send_add_new = _roster_send_add_new;
-    roster_send_remove = _roster_send_remove;
-    roster_send_name_change = _roster_send_name_change;
-    roster_send_add_to_group = _roster_send_add_to_group;
-    roster_send_remove_from_group = _roster_send_remove_from_group;
-
-}
+}
\ No newline at end of file
