@@ -3,5 +3,8 @@
 void load_preferences(void **state);
 void close_preferences(void **state);
 
+void init_chat_sessions(void **state);
+void close_chat_sessions(void **state);
+
 void glist_set_cmp(GCompareFunc func);
-int glist_contents_equal(const void *actual, const void *expected);
+int glist_contents_equal(const void *actual, const void *expected);
\ No newline at end of file
