@@ -7,22 +7,19 @@
 #include <glib.h>
 
 #include "xmpp/xmpp.h"
-#include "xmpp/mock_xmpp.h"
 
 #include "ui/ui.h"
-#include "ui/mock_ui.h"
+#include "ui/stub_ui.h"
 
 #include "config/accounts.h"
-#include "config/mock_accounts.h"
-
 #include "command/commands.h"
 
 static void test_with_connection_status(jabber_conn_status_t status)
 {
-    mock_cons_show();
     CommandHelp *help = malloc(sizeof(CommandHelp));
 
-    mock_connection_status(status);
+    will_return(jabber_get_connection_status, status);
+
     expect_cons_show("You are not currently connected.");
 
     gboolean result = cmd_rooms(NULL, *help);
@@ -58,13 +55,13 @@ void cmd_rooms_shows_message_when_undefined(void **state)
 
 void cmd_rooms_uses_account_default_when_no_arg(void **state)
 {
-    mock_accounts_get_account();
     CommandHelp *help = malloc(sizeof(CommandHelp));
     gchar *args[] = { NULL };
     ProfAccount *account = malloc(sizeof(ProfAccount));
     account->name = NULL;
     account->jid = NULL;
     account->password = NULL;
+    account->eval_password = NULL;
     account->resource = NULL;
     account->server = NULL;
     account->last_presence = NULL;
@@ -76,12 +73,12 @@ void cmd_rooms_uses_account_default_when_no_arg(void **state)
     account->otr_always = NULL;
     account->muc_service = strdup("default_conf_server");
 
-    mock_connection_status(JABBER_CONNECTED);
-    mock_connection_account_name("account_name");
+    will_return(jabber_get_connection_status, JABBER_CONNECTED);
+    will_return(jabber_get_account_name, "account_name");
+    expect_any(accounts_get_account, name);
+    will_return(accounts_get_account, account);
 
-    accounts_get_account_return(account);
-
-    expect_room_list_request("default_conf_server");
+    expect_string(iq_room_list_request, conferencejid, "default_conf_server");
 
     gboolean result = cmd_rooms(args, *help);
 
@@ -95,9 +92,9 @@ void cmd_rooms_arg_used_when_passed(void **state)
     CommandHelp *help = malloc(sizeof(CommandHelp));
     gchar *args[] = { "conf_server_arg" };
 
-    mock_connection_status(JABBER_CONNECTED);
+    will_return(jabber_get_connection_status, JABBER_CONNECTED);
 
-    expect_room_list_request("conf_server_arg");
+    expect_string(iq_room_list_request, conferencejid, "conf_server_arg");
 
     gboolean result = cmd_rooms(args, *help);
 
