@@ -8,9 +8,11 @@
 #include <sys/stat.h>
 
 #include "config.h"
+#include "chat_session.h"
 
 #include "helpers.h"
 #include "test_autocomplete.h"
+#include "test_chat_session.h"
 #include "test_common.h"
 #include "test_contact.h"
 #include "test_cmd_connect.h"
@@ -31,6 +33,7 @@
 #include "test_muc.h"
 #include "test_cmd_roster.h"
 #include "test_cmd_win.h"
+#include "test_cmd_disconnect.h"
 #include "test_form.h"
 
 int main(int argc, char* argv[]) {
@@ -82,6 +85,16 @@ int main(int argc, char* argv[]) {
         unit_test(test_p_sha1_hash5),
         unit_test(test_p_sha1_hash6),
         unit_test(test_p_sha1_hash7),
+        unit_test(utf8_display_len_null_str),
+        unit_test(utf8_display_len_1_non_wide),
+        unit_test(utf8_display_len_1_wide),
+        unit_test(utf8_display_len_non_wide),
+        unit_test(utf8_display_len_wide),
+        unit_test(utf8_display_len_all_wide),
+        unit_test(strip_quotes_does_nothing_when_no_quoted),
+        unit_test(strip_quotes_strips_first),
+        unit_test(strip_quotes_strips_last),
+        unit_test(strip_quotes_strips_both),
 
         unit_test(clear_empty),
         unit_test(reset_after_create),
@@ -131,6 +144,8 @@ int main(int argc, char* argv[]) {
         unit_test(create_with_at_in_resource),
         unit_test(create_with_at_and_slash_in_resource),
         unit_test(create_full_with_trailing_slash),
+        unit_test(returns_fulljid_when_exists),
+        unit_test(returns_barejid_when_fulljid_not_exists),
 
         unit_test(parse_null_returns_null),
         unit_test(parse_empty_returns_null),
@@ -202,31 +217,93 @@ int main(int argc, char* argv[]) {
         unit_test(find_five_times_finds_fifth),
         unit_test(find_twice_returns_first_when_two_match_and_reset),
 
-        unit_test(cmd_connect_shows_message_when_disconnecting),
-        unit_test(cmd_connect_shows_message_when_connecting),
-        unit_test(cmd_connect_shows_message_when_connected),
-        unit_test(cmd_connect_shows_message_when_undefined),
-        unit_test(cmd_connect_when_no_account),
-        unit_test(cmd_connect_fail_message),
-        unit_test(cmd_connect_lowercases_argument),
-        unit_test(cmd_connect_asks_password_when_not_in_account),
-        unit_test(cmd_connect_shows_message_when_connecting_with_account),
-        unit_test(cmd_connect_connects_with_account),
-        unit_test(cmd_connect_shows_usage_when_no_server_value),
-        unit_test(cmd_connect_shows_usage_when_server_no_port_value),
-        unit_test(cmd_connect_shows_usage_when_no_port_value),
-        unit_test(cmd_connect_shows_usage_when_port_no_server_value),
-        unit_test(cmd_connect_shows_message_when_port_0),
-        unit_test(cmd_connect_shows_message_when_port_minus1),
-        unit_test(cmd_connect_shows_message_when_port_65536),
-        unit_test(cmd_connect_shows_message_when_port_contains_chars),
-        unit_test(cmd_connect_with_server_when_provided),
-        unit_test(cmd_connect_with_port_when_provided),
-        unit_test(cmd_connect_with_server_and_port_when_provided),
-        unit_test(cmd_connect_shows_usage_when_server_provided_twice),
-        unit_test(cmd_connect_shows_usage_when_port_provided_twice),
-        unit_test(cmd_connect_shows_usage_when_invalid_first_property),
-        unit_test(cmd_connect_shows_usage_when_invalid_second_property),
+        unit_test_setup_teardown(returns_false_when_chat_session_does_not_exist,
+            init_chat_sessions,
+            close_chat_sessions),
+        unit_test_setup_teardown(creates_chat_session_on_recipient_activity,
+            init_chat_sessions,
+            close_chat_sessions),
+        unit_test_setup_teardown(replaces_chat_session_on_recipient_activity_with_different_resource,
+            init_chat_sessions,
+            close_chat_sessions),
+        unit_test_setup_teardown(removes_chat_session,
+            init_chat_sessions,
+            close_chat_sessions),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_disconnecting,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_connecting,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_connected,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_undefined,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_when_no_account,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_fail_message,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_lowercases_argument,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_asks_password_when_not_in_account,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_connecting_with_account,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_connects_with_account,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_no_server_value,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_server_no_port_value,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_no_port_value,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_port_no_server_value,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_port_0,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_port_minus1,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_port_65536,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_message_when_port_contains_chars,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_with_server_when_provided,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_with_port_when_provided,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_with_server_and_port_when_provided,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_server_provided_twice,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_port_provided_twice,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_invalid_first_property,
+            load_preferences,
+            close_preferences),
+        unit_test_setup_teardown(cmd_connect_shows_usage_when_invalid_second_property,
+            load_preferences,
+            close_preferences),
 
         unit_test(cmd_rooms_shows_message_when_disconnected),
         unit_test(cmd_rooms_shows_message_when_disconnecting),
@@ -244,43 +321,37 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_account_show_shows_account_when_exists),
         unit_test(cmd_account_add_shows_usage_when_no_arg),
         unit_test(cmd_account_add_adds_account),
-        unit_test(cmd_account_add_shows_message),
         unit_test(cmd_account_enable_shows_usage_when_no_arg),
         unit_test(cmd_account_enable_enables_account),
-        unit_test(cmd_account_enable_shows_message_when_enabled),
         unit_test(cmd_account_enable_shows_message_when_account_doesnt_exist),
         unit_test(cmd_account_disable_shows_usage_when_no_arg),
         unit_test(cmd_account_disable_disables_account),
-        unit_test(cmd_account_disable_shows_message_when_disabled),
         unit_test(cmd_account_disable_shows_message_when_account_doesnt_exist),
         unit_test(cmd_account_rename_shows_usage_when_no_args),
         unit_test(cmd_account_rename_shows_usage_when_one_arg),
         unit_test(cmd_account_rename_renames_account),
-        unit_test(cmd_account_rename_shows_message_when_renamed),
         unit_test(cmd_account_rename_shows_message_when_not_renamed),
         unit_test(cmd_account_set_shows_usage_when_no_args),
         unit_test(cmd_account_set_shows_usage_when_one_arg),
         unit_test(cmd_account_set_shows_usage_when_two_args),
-        unit_test(cmd_account_set_checks_account_exists),
         unit_test(cmd_account_set_shows_message_when_account_doesnt_exist),
         unit_test(cmd_account_set_jid_shows_message_for_malformed_jid),
         unit_test(cmd_account_set_jid_sets_barejid),
         unit_test(cmd_account_set_jid_sets_resource),
         unit_test(cmd_account_set_server_sets_server),
-        unit_test(cmd_account_set_server_shows_message),
         unit_test(cmd_account_set_resource_sets_resource),
-        unit_test(cmd_account_set_resource_shows_message),
         unit_test(cmd_account_set_password_sets_password),
-        unit_test(cmd_account_set_password_shows_message),
+        unit_test(cmd_account_set_eval_password_sets_eval_password),
+        unit_test(cmd_account_set_password_when_eval_password_set),
+        unit_test(cmd_account_set_eval_password_when_password_set),
         unit_test(cmd_account_set_muc_sets_muc),
-        unit_test(cmd_account_set_muc_shows_message),
         unit_test(cmd_account_set_nick_sets_nick),
-        unit_test(cmd_account_set_nick_shows_message),
+        unit_test(cmd_account_show_message_for_missing_otr_policy),
+        unit_test(cmd_account_show_message_for_invalid_otr_policy),
+        unit_test(cmd_account_set_otr_sets_otr),
         unit_test(cmd_account_set_status_shows_message_when_invalid_status),
         unit_test(cmd_account_set_status_sets_status_when_valid),
         unit_test(cmd_account_set_status_sets_status_when_last),
-        unit_test(cmd_account_set_status_shows_message_when_set_valid),
-        unit_test(cmd_account_set_status_shows_message_when_set_last),
         unit_test(cmd_account_set_invalid_presence_string_priority_shows_message),
         unit_test(cmd_account_set_last_priority_shows_message),
         unit_test(cmd_account_set_online_priority_sets_preference),
@@ -288,7 +359,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_account_set_away_priority_sets_preference),
         unit_test(cmd_account_set_xa_priority_sets_preference),
         unit_test(cmd_account_set_dnd_priority_sets_preference),
-        unit_test(cmd_account_set_online_priority_shows_message),
         unit_test(cmd_account_set_priority_too_low_shows_message),
         unit_test(cmd_account_set_priority_too_high_shows_message),
         unit_test(cmd_account_set_priority_when_not_number_shows_message),
@@ -296,7 +366,6 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_account_set_priority_updates_presence_when_account_connected_with_presence),
         unit_test(cmd_account_clear_shows_usage_when_no_args),
         unit_test(cmd_account_clear_shows_usage_when_one_arg),
-        unit_test(cmd_account_clear_checks_account_exists),
         unit_test(cmd_account_clear_shows_message_when_account_doesnt_exist),
         unit_test(cmd_account_clear_shows_message_when_invalid_property),
 
@@ -398,6 +467,8 @@ int main(int argc, char* argv[]) {
         unit_test(handle_message_error_when_recipient_and_no_type),
         unit_test(handle_presence_error_when_no_recipient),
         unit_test(handle_presence_error_when_from_recipient),
+        unit_test(handle_offline_removes_chat_session),
+        unit_test(lost_connection_clears_chat_sessions),
 
         unit_test(cmd_alias_add_shows_usage_when_no_args),
         unit_test(cmd_alias_add_shows_usage_when_no_value),
@@ -494,10 +565,13 @@ int main(int argc, char* argv[]) {
         unit_test(cmd_otr_start_shows_message_when_in_private),
         unit_test(cmd_otr_start_shows_message_when_already_started),
         unit_test(cmd_otr_start_shows_message_when_no_key),
-        unit_test(cmd_otr_start_sends_otr_query_message_to_current_recipeint),
+        unit_test_setup_teardown(cmd_otr_start_sends_otr_query_message_to_current_recipeint,
+            load_preferences,
+            close_preferences),
 #else
         unit_test(cmd_otr_shows_message_when_otr_unsupported),
 #endif
+
         unit_test(cmd_join_shows_message_when_disconnecting),
         unit_test(cmd_join_shows_message_when_connecting),
         unit_test(cmd_join_shows_message_when_disconnected),
@@ -550,6 +624,8 @@ int main(int argc, char* argv[]) {
         unit_test(remove_text_multi_value_does_nothing_when_doesnt_exist),
         unit_test(remove_text_multi_value_removes_when_one),
         unit_test(remove_text_multi_value_removes_when_many),
+
+        unit_test(clears_chat_sessions),
     };
 
     return run_tests(all_tests);
